Index: files.el
===================================================================
RCS file: /usr/CVSroot/XEmacs/xemacs/lisp/files.el,v
retrieving revision 1.39.2.2
diff -u -r1.39.2.2 files.el
--- files.el	23 Oct 2001 08:35:21 -0000	1.39.2.2
+++ files.el	31 Oct 2001 21:18:08 -0000
@@ -159,6 +159,15 @@
   :type '(repeat (regexp ""))
   :group 'find-file)
 
+(defcustom revert-uses-md5-equality t
+  "*If t then don't re-read files that are identical to the current buffer.
+Revert-buffer can get very expensive, and often with source-code
+control systems the file on disk is actually identical to the one in
+the buffer. This optimization uses md5 to determine equality of the
+buffer with the file on disk." 
+  :type 'boolean
+  :group 'find-file)
+
 (defvar buffer-file-number nil
   "The device number and file number of the file visited in the current buffer.
 The value is a list of the form (FILENUM DEVNUM).
@@ -2709,6 +2718,22 @@
 			buffer-file-name)))
       (cond ((null file-name)
 	     (error "Buffer does not seem to be associated with any file"))
+	    ;; revert-buffer can get very expensive (setting modes,
+	    ;; font-lock etc.) and often the file on disk is actually
+	    ;; identical to the one in the buffer. This optimization
+	    ;; use md5 to determine equality of the buffer with the
+	    ;; file on disk.
+	    ((and revert-uses-md5-equality
+		  (not (buffer-modified-p))
+		  ;; No edits and buffer on disk is the same.
+		  (or (verify-visited-file-modtime (current-buffer))
+		      (let ((buf (current-buffer)))
+			(with-temp-buffer
+			  (insert-file-contents file-name)
+			  (equal (md5 (current-buffer))
+				 (md5 buf))))))
+	     (set-visited-file-modtime)
+	     (setq buffer-read-only (not (file-writable-p buffer-file-name))))
 	    ((or noconfirm
 		 (and (not (buffer-modified-p))
 		      (let (found)
